@@ -26,7 +26,7 @@
  */
 
 #include "config.h"
-#include "version.h"
+#include "vcs_version.h"
 
 #include <errno.h>
 #include <string.h>
@@ -38,6 +38,7 @@
 #include "common/validate.h"
 
 #include "src/internal.h"
+#include "src/log.h"
 #include "src/obu.h"
 #include "src/qm.h"
 #include "src/ref.h"
@@ -62,10 +63,14 @@ void dav1d_default_settings(Dav1dSettings *const s) {
     s->allocator.cookie = NULL;
     s->allocator.alloc_picture_callback = default_picture_allocator;
     s->allocator.release_picture_callback = default_picture_release;
+    s->logger.cookie = NULL;
+    s->logger.callback = dav1d_log_default_callback;
     s->operating_point = 0;
     s->all_layers = 1; // just until the tests are adjusted
 }
 
+static void close_internal(Dav1dContext **const c_out, int flush);
+
 int dav1d_open(Dav1dContext **const c_out,
                const Dav1dSettings *const s)
 {
@@ -90,6 +95,7 @@ int dav1d_open(Dav1dContext **const c_out,
     memset(c, 0, sizeof(*c));
 
     c->allocator = s->allocator;
+    c->logger = s->logger;
     c->apply_grain = s->apply_grain;
     c->operating_point = s->operating_point;
     c->all_layers = s->all_layers;
@@ -115,9 +121,17 @@ int dav1d_open(Dav1dContext **const c_out,
         if (!f->tc) goto error;
         memset(f->tc, 0, sizeof(*f->tc) * s->n_tile_threads);
         if (f->n_tc > 1) {
-            pthread_mutex_init(&f->tile_thread.lock, NULL);
-            pthread_cond_init(&f->tile_thread.cond, NULL);
-            pthread_cond_init(&f->tile_thread.icond, NULL);
+            if (pthread_mutex_init(&f->tile_thread.lock, NULL)) goto error;
+            if (pthread_cond_init(&f->tile_thread.cond, NULL)) {
+                pthread_mutex_destroy(&f->tile_thread.lock);
+                goto error;
+            }
+            if (pthread_cond_init(&f->tile_thread.icond, NULL)) {
+                pthread_mutex_destroy(&f->tile_thread.lock);
+                pthread_cond_destroy(&f->tile_thread.cond);
+                goto error;
+            }
+            f->tile_thread.inited = 1;
         }
         for (int m = 0; m < s->n_tile_threads; m++) {
             Dav1dTileContext *const t = &f->tc[m];
@@ -131,18 +145,34 @@ int dav1d_open(Dav1dContext **const c_out,
                 dav1d_alloc_aligned(320 * (256 + 7) * sizeof(uint16_t), 32);
             if (!t->emu_edge) goto error;
             if (f->n_tc > 1) {
-                pthread_mutex_init(&t->tile_thread.td.lock, NULL);
-                pthread_cond_init(&t->tile_thread.td.cond, NULL);
+                if (pthread_mutex_init(&t->tile_thread.td.lock, NULL)) goto error;
+                if (pthread_cond_init(&t->tile_thread.td.cond, NULL)) {
+                    pthread_mutex_destroy(&t->tile_thread.td.lock);
+                    goto error;
+                }
                 t->tile_thread.fttd = &f->tile_thread;
-                pthread_create(&t->tile_thread.td.thread, NULL, dav1d_tile_task, t);
+                if (pthread_create(&t->tile_thread.td.thread, NULL, dav1d_tile_task, t)) {
+                    pthread_cond_destroy(&t->tile_thread.td.cond);
+                    pthread_mutex_destroy(&t->tile_thread.td.lock);
+                    goto error;
+                }
+                t->tile_thread.td.inited = 1;
             }
         }
         f->libaom_cm = av1_alloc_ref_mv_common();
         if (!f->libaom_cm) goto error;
         if (c->n_fc > 1) {
-            pthread_mutex_init(&f->frame_thread.td.lock, NULL);
-            pthread_cond_init(&f->frame_thread.td.cond, NULL);
-            pthread_create(&f->frame_thread.td.thread, NULL, dav1d_frame_task, f);
+            if (pthread_mutex_init(&f->frame_thread.td.lock, NULL)) goto error;
+            if (pthread_cond_init(&f->frame_thread.td.cond, NULL)) {
+                pthread_mutex_destroy(&f->frame_thread.td.lock);
+                goto error;
+            }
+            if (pthread_create(&f->frame_thread.td.thread, NULL, dav1d_frame_task, f)) {
+                pthread_cond_destroy(&f->frame_thread.td.cond);
+                pthread_mutex_destroy(&f->frame_thread.td.lock);
+                goto error;
+            }
+            f->frame_thread.td.inited = 1;
         }
     }
 
@@ -155,26 +185,7 @@ int dav1d_open(Dav1dContext **const c_out,
     return 0;
 
 error:
-    if (c) {
-        if (c->fc) {
-            for (unsigned n = 0; n < c->n_fc; n++) {
-                if (c->fc[n].tc) {
-                    for (int m = 0; m < s->n_tile_threads; m++) {
-                        Dav1dTileContext *const t = &c->fc[n].tc[m];
-                        dav1d_free_aligned(t->cf);
-                        dav1d_free_aligned(t->scratch.mem);
-                        dav1d_free_aligned(t->emu_edge);
-                    }
-                    dav1d_free_aligned(c->fc[n].tc);
-                }
-                if (c->fc[n].libaom_cm) av1_free_ref_mv_common(c->fc[n].libaom_cm);
-            }
-            dav1d_free_aligned(c->fc);
-        }
-        if (c->n_fc > 1) free(c->frame_thread.out_delayed);
-        dav1d_freep_aligned(c_out);
-    }
-    fprintf(stderr, "Failed to allocate memory: %s\n", strerror(errno));
+    if (c) close_internal(c_out, 0);
     return -ENOMEM;
 }
 
@@ -192,13 +203,14 @@ int dav1d_parse_sequence_header(Dav1dSequenceHeader *const out,
 
     Dav1dSettings s;
     dav1d_default_settings(&s);
+    s.logger.callback = NULL;
 
     Dav1dContext *c;
-    res	= dav1d_open(&c, &s);
+    res = dav1d_open(&c, &s);
     if (res < 0) return res;
 
     if (ptr) {
-        res = dav1d_data_wrap(&buf, ptr, sz, dummy_free, NULL);
+        res = dav1d_data_wrap_internal(&buf, ptr, sz, dummy_free, NULL);
         if (res < 0) goto error;
     }
 
@@ -220,7 +232,7 @@ int dav1d_parse_sequence_header(Dav1dSequenceHeader *const out,
 
     res = 0;
 error:
-    dav1d_data_unref(&buf);
+    dav1d_data_unref_internal(&buf);
     dav1d_close(&c);
 
     return res;
@@ -255,10 +267,10 @@ static int output_image(Dav1dContext *const c, Dav1dPicture *const out,
     }
 
     // Apply film grain to a new copy of the image to avoid corrupting refs
-    int res = dav1d_picture_alloc_copy(out, in->p.w, in);
+    int res = dav1d_picture_alloc_copy(c, out, in->p.w, in);
     if (res < 0) {
-        dav1d_picture_unref(in);
-        dav1d_picture_unref(out);
+        dav1d_picture_unref_internal(in);
+        dav1d_picture_unref_internal(out);
         return res;
     }
 
@@ -278,7 +290,7 @@ static int output_image(Dav1dContext *const c, Dav1dPicture *const out,
         assert(0);
     }
 
-    dav1d_picture_unref(in);
+    dav1d_picture_unref_internal(in);
     return 0;
 }
 
@@ -290,7 +302,7 @@ static int output_picture_ready(Dav1dContext *const c) {
     if (c->operating_point_idc && !c->all_layers) {
         const int max_spatial_id = ulog2(c->operating_point_idc >> 8);
         if (max_spatial_id > c->out.frame_hdr->spatial_id) {
-            dav1d_picture_unref(&c->out);
+            dav1d_picture_unref_internal(&c->out);
             return 0;
         }
     }
@@ -346,12 +358,12 @@ int dav1d_get_picture(Dav1dContext *const c, Dav1dPicture *const out)
     while (in->sz > 0) {
         res = dav1d_parse_obus(c, in, 0);
         if (res < 0) {
-            dav1d_data_unref(in);
+            dav1d_data_unref_internal(in);
         } else {
             assert((size_t)res <= in->sz);
             in->sz -= res;
             in->data += res;
-            if (!in->sz) dav1d_data_unref(in);
+            if (!in->sz) dav1d_data_unref_internal(in);
         }
         if (output_picture_ready(c))
             break;
@@ -369,9 +381,25 @@ int dav1d_get_picture(Dav1dContext *const c, Dav1dPicture *const out)
 }
 
 void dav1d_flush(Dav1dContext *const c) {
-    dav1d_data_unref(&c->in);
+    dav1d_data_unref_internal(&c->in);
     c->drain = 0;
 
+    for (int i = 0; i < 8; i++) {
+        if (c->refs[i].p.p.data[0])
+            dav1d_thread_picture_unref(&c->refs[i].p);
+        dav1d_ref_dec(&c->refs[i].segmap);
+        dav1d_ref_dec(&c->refs[i].refmvs);
+        dav1d_cdf_thread_unref(&c->cdf[i]);
+    }
+    c->frame_hdr = NULL;
+    c->seq_hdr = NULL;
+    dav1d_ref_dec(&c->seq_hdr_ref);
+
+    c->mastering_display = NULL;
+    c->content_light = NULL;
+    dav1d_ref_dec(&c->mastering_display_ref);
+    dav1d_ref_dec(&c->content_light_ref);
+
     if (c->n_fc == 1) return;
 
     // mark each currently-running frame as flushing, so that we
@@ -394,32 +422,25 @@ void dav1d_flush(Dav1dContext *const c) {
     }
     atomic_store(c->frame_thread.flush, 0);
 
-    for (int i = 0; i < 8; i++) {
-        if (c->refs[i].p.p.data[0])
-            dav1d_thread_picture_unref(&c->refs[i].p);
-        dav1d_ref_dec(&c->refs[i].segmap);
-        dav1d_ref_dec(&c->refs[i].refmvs);
-        dav1d_cdf_thread_unref(&c->cdf[i]);
-    }
-    c->frame_hdr = NULL;
-    c->seq_hdr = NULL;
-    dav1d_ref_dec(&c->seq_hdr_ref);
-
     c->frame_thread.next = 0;
 }
 
 void dav1d_close(Dav1dContext **const c_out) {
     validate_input(c_out != NULL);
+    close_internal(c_out, 1);
+}
 
+static void close_internal(Dav1dContext **const c_out, int flush) {
     Dav1dContext *const c = *c_out;
     if (!c) return;
 
-    dav1d_flush(c);
-    for (unsigned n = 0; n < c->n_fc; n++) {
+    if (flush) dav1d_flush(c);
+
+    for (unsigned n = 0; c->fc && n < c->n_fc; n++) {
         Dav1dFrameContext *const f = &c->fc[n];
 
         // clean-up threading stuff
-        if (c->n_fc > 1) {
+        if (c->n_fc > 1 && f->frame_thread.td.inited) {
             pthread_mutex_lock(&f->frame_thread.td.lock);
             f->frame_thread.die = 1;
             pthread_cond_signal(&f->frame_thread.td.cond);
@@ -429,16 +450,19 @@ void dav1d_close(Dav1dContext **const c_out) {
             dav1d_freep_aligned(&f->frame_thread.pal_idx);
             dav1d_freep_aligned(&f->frame_thread.cf);
             freep(&f->frame_thread.tile_start_off);
-            freep(&f->frame_thread.pal);
+            dav1d_freep_aligned(&f->frame_thread.pal);
             freep(&f->frame_thread.cbi);
             pthread_mutex_destroy(&f->frame_thread.td.lock);
             pthread_cond_destroy(&f->frame_thread.td.cond);
         }
-        if (f->n_tc > 1) {
+        if (f->n_tc > 1 && f->tc && f->tile_thread.inited) {
             pthread_mutex_lock(&f->tile_thread.lock);
             for (int m = 0; m < f->n_tc; m++) {
                 Dav1dTileContext *const t = &f->tc[m];
                 t->tile_thread.die = 1;
+                // mark not created tile threads as available
+                if (!t->tile_thread.td.inited)
+                    f->tile_thread.available |= 1ULL<<m;
             }
             pthread_cond_broadcast(&f->tile_thread.cond);
             while (f->tile_thread.available != ~0ULL >> (64 - f->n_tc))
@@ -447,7 +471,7 @@ void dav1d_close(Dav1dContext **const c_out) {
             pthread_mutex_unlock(&f->tile_thread.lock);
             for (int m = 0; m < f->n_tc; m++) {
                 Dav1dTileContext *const t = &f->tc[m];
-                if (f->n_tc > 1) {
+                if (f->n_tc > 1 && t->tile_thread.td.inited) {
                     pthread_join(t->tile_thread.td.thread, NULL);
                     pthread_mutex_destroy(&t->tile_thread.td.lock);
                     pthread_cond_destroy(&t->tile_thread.td.cond);
@@ -458,13 +482,13 @@ void dav1d_close(Dav1dContext **const c_out) {
             pthread_cond_destroy(&f->tile_thread.icond);
             freep(&f->tile_thread.task_idx_to_sby_and_tile_idx);
         }
-        for (int m = 0; m < f->n_tc; m++) {
+        for (int m = 0; f->tc && m < f->n_tc; m++) {
             Dav1dTileContext *const t = &f->tc[m];
             dav1d_free_aligned(t->cf);
             dav1d_free_aligned(t->scratch.mem);
             dav1d_free_aligned(t->emu_edge);
         }
-        for (int m = 0; m < f->n_ts; m++) {
+        for (int m = 0; f->ts && m < f->n_ts; m++) {
             Dav1dTileState *const ts = &f->ts[m];
             pthread_cond_destroy(&ts->tile_thread.cond);
             pthread_mutex_destroy(&ts->tile_thread.lock);
@@ -473,24 +497,26 @@ void dav1d_close(Dav1dContext **const c_out) {
         dav1d_free_aligned(f->tc);
         dav1d_free_aligned(f->ipred_edge[0]);
         free(f->a);
+        free(f->tile);
         free(f->lf.mask);
         free(f->lf.lr_mask);
         free(f->lf.level);
         free(f->lf.tx_lpf_right_edge[0]);
-        av1_free_ref_mv_common(f->libaom_cm);
+        if (f->libaom_cm) av1_free_ref_mv_common(f->libaom_cm);
         dav1d_free_aligned(f->lf.cdef_line);
         dav1d_free_aligned(f->lf.lr_lpf_line);
     }
     dav1d_free_aligned(c->fc);
-    dav1d_data_unref(&c->in);
-    if (c->n_fc > 1) {
+    dav1d_data_unref_internal(&c->in);
+    if (c->n_fc > 1 && c->frame_thread.out_delayed) {
         for (unsigned n = 0; n < c->n_fc; n++)
             if (c->frame_thread.out_delayed[n].p.data[0])
                 dav1d_thread_picture_unref(&c->frame_thread.out_delayed[n]);
         free(c->frame_thread.out_delayed);
     }
     for (int n = 0; n < c->n_tile_data; n++)
-        dav1d_data_unref(&c->tile[n].data);
+        dav1d_data_unref_internal(&c->tile[n].data);
+    free(c->tile);
     for (int n = 0; n < 8; n++) {
         dav1d_cdf_thread_unref(&c->cdf[n]);
         if (c->refs[n].p.p.data[0])
@@ -501,5 +527,41 @@ void dav1d_close(Dav1dContext **const c_out) {
     dav1d_ref_dec(&c->seq_hdr_ref);
     dav1d_ref_dec(&c->frame_hdr_ref);
 
+    dav1d_ref_dec(&c->mastering_display_ref);
+    dav1d_ref_dec(&c->content_light_ref);
+
     dav1d_freep_aligned(c_out);
 }
+
+void dav1d_picture_unref(Dav1dPicture *const p) {
+    dav1d_picture_unref_internal(p);
+}
+
+uint8_t *dav1d_data_create(Dav1dData *const buf, const size_t sz) {
+    return dav1d_data_create_internal(buf, sz);
+}
+
+int dav1d_data_wrap(Dav1dData *const buf, const uint8_t *const ptr,
+                    const size_t sz,
+                    void (*const free_callback)(const uint8_t *data,
+                                                void *user_data),
+                    void *const user_data)
+{
+    return dav1d_data_wrap_internal(buf, ptr, sz, free_callback, user_data);
+}
+
+int dav1d_data_wrap_user_data(Dav1dData *const buf,
+                              const uint8_t *const user_data,
+                              void (*const free_callback)(const uint8_t *user_data,
+                                                          void *cookie),
+                              void *const cookie)
+{
+    return dav1d_data_wrap_user_data_internal(buf,
+                                              user_data,
+                                              free_callback,
+                                              cookie);
+}
+
+void dav1d_data_unref(Dav1dData *const buf) {
+    dav1d_data_unref_internal(buf);
+}
